module Segurocomar
  class Api
    def initialize(args={})
      @access_token = args[:access_token]
      @endpoint_url = args[:endpoint_url]
      @debug = args[:debug]
    end

    include Segurocomar::Core::ArtActivities
    include Segurocomar::Core::Bicycles
    include Segurocomar::Core::Cars
    include Segurocomar::Core::Insurance
    include Segurocomar::Core::Misc
    include Segurocomar::Core::Motorcycles
    include Segurocomar::Core::Quotes
    include Segurocomar::Core::Trucks

    private

    def get_request(action, params={}, headers={})
      headers.merge!({ authorization: "Token token=\"#{@access_token}\"" })

      begin
        parse_response(RestClient.get("#{@endpoint_url}#{action}", {params: params}.merge(headers)))
      rescue => e
        parse_response(e.response)
      end
    end

    def post_request(action, params={}, headers={})
      headers.merge!({ authorization: "Token token=\"#{@access_token}\"" })

      begin
        parse_response(RestClient.post("#{@endpoint_url}#{action}", params, headers))
      rescue => e
        parse_response(e.response)
      end
    end

    def put_request(action, params={}, headers={})
      headers.merge!({ authorization: "Token token=\"#{@access_token}\"" })

      begin
        parse_response(RestClient.put("#{@endpoint_url}#{action}", params, headers))
      rescue => e
        parse_response(e.response)
      end
    end

    def patch_request(action, params={}, headers={})
      headers.merge!({ authorization: "Token token=\"#{@access_token}\"" })

      begin
        parse_response(RestClient.patch("#{@endpoint_url}#{action}", params, headers))
      rescue => e
        parse_response(e.response)
      end
    end

    def head_request(action, params={})
      headers.merge!({ authorization: "Token token=\"#{@access_token}\"" })

      begin
        parse_response(RestClient.head("#{@endpoint_url}#{action}", params))
      rescue => e
        parse_response(e.response)
      end
    end

    def delete_request(action, params={})
      headers.merge!({ authorization: "Token token=\"#{@access_token}\"" })

      begin
        parse_response(RestClient.delete("#{@endpoint_url}#{action}", params))
      rescue => e
        parse_response(e.response)
      end
    end

    def parse_response(response)
      result = {
        headers: response.headers,
        body: (JSON.parse(response.body) rescue response.body),
        status_code: response.code
      }

      p "DEBUG: #{result}" if @debug

      result
    end
  end
end
